#ifndef __AMSCLIL1_ULINT_HPP__
#define __AMSCLIL1_ULINT_HPP__

///////////////////////////////////
// Unsigned Large Integer Object //
///////////////////////////////////

// This is an unsigned large integer object based on 32-bit words. 32 bits seems to be
// the fastest width for the underlying data.

typedef struct amsclil1_ulint
{
    int N;
    uint32_t *data;
} amsclil1_ulint;

// Container Class Routines

int amsclil1_ulint_new(amsclil1_ulint **pnt);
void amsclil1_ulint_delete(amsclil1_ulint **pnt);

int amsclil1_ulint_init(amsclil1_ulint *pnt);
void amsclil1_ulint_cleanup(amsclil1_ulint *pnt);

//resize integer buffer to new size _N
int amsclil1_ulint_resize(amsclil1_ulint *pnt, int _N);

//If _N is greater than current size, expand to larger size, otherwise do nothing
int amsclil1_ulint_resizetomin(amsclil1_ulint *pnt, int _N);

//Most significant index of number
int amsclil1_ulint_msindex(amsclil1_ulint *pnt);

//shrink memory to most significant index
int amsclil1_ulint_shrinktofit(amsclil1_ulint *pnt);

//sets all data to 0
void amsclil1_ulint_clear(amsclil1_ulint *pnt);

//copies src data to dest. resizes dest if necessary, otherwise does not.
int amsclil1_ulint_copy(amsclil1_ulint *dest, amsclil1_ulint *src);

// Display

void amsclil1_ulint_printhex(amsclil1_ulint *pnt);
void amsclil1_ulint_printbits(amsclil1_ulint *pnt);

// Conversion To/From amscutil bytebuffer


// Comparators

// Unsigned Large Integer Comparison Operator: 
// 0 - a==b
// 1 - a>b
// 2 - a<b
int amsclil1_ulint_cmp(amsclil1_ulint *a, amsclil1_ulint *b);

int amsclil1_ulint_iszero(amsclil1_ulint *a);

// Math Operations
//      These operations may be slower than the preallocated fixed buffer operations
//      They are included for simplicity in constructing and testing algorithms.
//      The necessary sizes and working memory is preallocated during execution.
//      Inputs are copied internally, so output can point to the same integer as input.

int amsclil1_ulint_add(amsclil1_ulint *a, amsclil1_ulint *b, amsclil1_ulint *c);
int amsclil1_ulint_sub(amsclil1_ulint *a, amsclil1_ulint *b, amsclil1_ulint *c);
int amsclil1_ulint_mult(amsclil1_ulint *a, amsclil1_ulint *b, amsclil1_ulint *c);
//int amsclil1_ulint_div(amsclil1_ulint *a, amsclil1_ulint *b, amsclil1_ulint *q, amsclil1_ulint *r);
//modpower
//rep_modpower
//gcd
//lcm


//Tests

void amsclil1_ulint_tests1();

#endif

