#include <amsclil1/amsclil1.h>

void test_fbops_printcmp()
{
    long N = 2;
    uint32_t a[N];
    uint32_t b[N];
    int c0;

    printf("test_fbops_printcmp\n");

    amsc_ui32_memset(a,0,N);
    amsc_ui32_memset(b,0,N);
    printf("a="); amsclil1_fb32_liprint(a,N); printf("\n");
    c0 = amsclil1_fb32_iszero(a,N);
    printf("a is zero?: %d\n",c0); 
    a[0] = 0x000000ff;
    printf("a="); amsclil1_fb32_liprint(a,N); printf("\n");
    c0 = amsclil1_fb32_iszero(a,N);
    printf("a is zero?: %d\n",c0);

    printf("\n");
    printf("a="); amsclil1_fb32_liprint(a,N); printf("\n"); 
    printf("b="); amsclil1_fb32_liprint(b,N); printf("\n");
    c0 = amsclil1_fb32_cmp(a,b,N);
    printf("cmp(a,b): %d\n",c0); 

    printf("\n");
    b[0] = 0x000000ff;
    printf("a="); amsclil1_fb32_liprint(a,N); printf("\n"); 
    printf("b="); amsclil1_fb32_liprint(b,N); printf("\n");
    c0 = amsclil1_fb32_cmp(a,b,N);
    printf("cmp(a,b): %d\n",c0); 

    printf("\n");
    b[0] = 0x00000fff;
    printf("a="); amsclil1_fb32_liprint(a,N); printf("\n"); 
    printf("b="); amsclil1_fb32_liprint(b,N); printf("\n");
    printf("b="); amsclil1_fb32_libinprint(b,N); printf("\n");
    c0 = amsclil1_fb32_cmp(a,b,N);
    printf("cmp(a,b): %d\n",c0); 

    b[0] = 0b00000000111111110000000011111111; //checking to see whether C understands binary constants
    printf("b="); amsclil1_fb32_libinprint(b,N); printf("\n");

    fflush(stdout);
    return;
}

void amsclil1_intl_printbin(uint32_t b)
{
    int I;
    uint32_t bit;
    for(I=32-1;I>=0;I--)
    {
        bit = (b & (1<<I)) >> I;
        printf("%d",bit);
    }
}

void amsclil1_intl_printshift(uint32_t *a, long shift, long blen)
{
    long I;
    uint32_t r;
    printf("0b:");
    for(I=blen-1;I>=0;I--)
    {
        amsclil1_fb32_readshift(a,I,shift,&r,blen);
        amsclil1_intl_printbin(r);
        if(I!=0) printf(":");
    }
}

void test_fbops_shiftread()
{
    long N=2;
    uint32_t a[N];
    long I;
    long shift;
    uint32_t r;

    amsc_ui32_memset(a,0b00001111000011110000111100001111,N);
    a[0] = 0b01010101010101010101010101010101;

    for(I=0;I<35;I++)
    {
        //something wrong when I=32
        printf("a<<%3ld= ",I);
        amsclil1_intl_printshift(a,I,N);
        printf("\n");
    }

    for(I=35;I>-35;I--)
    {
        printf("a<<%3ld= ",I);
        amsclil1_intl_printshift(a,I,N);
        printf("\n");
    }


    return;
}

void test_fbops_shiftwrite()
{
    long N = 2;
    uint32_t a[N];
    uint32_t r;
    long I;

    printf("hello?");
    for(I=-40;I<40;I++)
    {
        //a[1] = 0b11111111111111111111111111111111;
        //a[0] = 0b00000000000000000000000000000000;
        
        a[0] = 0b11110000111100001111000011110000;
        a[1] = 0b11111111000000001111111100000000;
        //printf("a="); amsclil1_fb32_libinprint(a,N); printf("\n");
        r =    0b11100011111111111111111111000111;
        amsclil1_fb32_writeshift(a,0,I,&r,N);
        printf("a w%3ld=",I); amsclil1_fb32_libinprint(a,N); printf("\n");
    }

    return;
}

void test_modtdiv()
{
    long I,J,K;

    printf("Tests of modular arithmetic function and true division.\n");

    J = 5;
    for(I=-20;I<20;I++)
    {
        K = amsc_ltdiv(I,J);
        printf("%ld / %ld = %ld\n",I,J,K);    
    }

    J = 5;
    for(I=-20;I<20;I++)
    {
        K = amsc_lmod(I,J);
        printf("%ld %% %ld = %ld\n",I,J,K);    
    }


    return;
}

void test_fbops_lshiftrshift()
{
    long N = 2;
    uint32_t a[N],b[N];
    long I;
    uint32_t r;

    printf("Test lshift and rshift.\n");

    a[1] = 0xff00ffff;
    a[0] = 0xf0f0ff00;

    for(I=0;I<35;I++)
    {
        printf("a="); amsclil1_fb32_libinprint(a,N); printf("\n");
        amsclil1_fb32_shiftl(a,1,b,N);
    }

    for(I=0;I<35;I++)
    {
        printf("a="); amsclil1_fb32_libinprint(a,N); printf("\n");
        amsclil1_fb32_shiftr(a,1,b,N);
    }

    a[1] = 0xff00ffff;
    a[0] = 0xf0f0ff00;
    for(I=0;I<2;I++)
    {
        printf("a="); amsclil1_fb32_libinprint(a,N); printf("\n");
        amsclil1_fb32_shiftl(a,32,b,N);
    }

    a[1] = 0xff00ffff;
    a[0] = 0xf0f0ff00;
    for(I=0;I<8;I++)
    {
        printf("a="); amsclil1_fb32_libinprint(a,N); printf("\n");
        amsclil1_fb32_shiftl(a,8,b,N);
    }

    return;
}