#include <amsclil1/amsclil1.h>

int amsclil1_ulint_new(amsclil1_ulint **pnt)
{
    int ret = 0;
    if(*pnt!=NULL) amsclil1_ulint_delete(pnt);

    *pnt = malloc(sizeof(amsclil1_ulint));

    (*pnt)->N = 0;
    (*pnt)->data = malloc(sizeof(uint32_t)*1);
    if((*pnt)->data==NULL)
    {
        ret = -1;
        return ret;
    }
    
    ret = 1;
    (*pnt)->data[0] = 0;

    return ret;
}

void amsclil1_ulint_delete(amsclil1_ulint **pnt)
{
    if(*pnt==NULL) return;
    
    if((*pnt)->data!=NULL) {free((*pnt)->data); (*pnt)->data=NULL;}
    (*pnt)->N = 0;

    free(*pnt); *pnt=NULL;

    return;
}

int amsclil1_ulint_init(amsclil1_ulint *pnt)
{
    int ret = 0;
    if(pnt==NULL)
    {
        ret = -1; return ret;
    }

    amsclil1_ulint_cleanup(pnt);
    pnt->N = 0;
    pnt->data = malloc(sizeof(uint32_t)*1);
    pnt->data[0] = 0;

    if(pnt->data==NULL)
    {
        ret = -1;
    }
    else
    {
        ret = 1;
    }
    return ret;
}

void amsclil1_ulint_cleanup(amsclil1_ulint *pnt)
{
    if(pnt!=NULL)
    {
        if(pnt->data!=NULL) {free(pnt->data); pnt->data=NULL;}
        pnt->N = 0;
    }
    return;
}

int amsclil1_ulint_resize(amsclil1_ulint *pnt, int _N)
{
    int ret = 0;
    int I = 0;
    uint32_t *newdata = NULL;


    if(pnt==NULL) {ret = -1; return ret;}
    
    if(_N<=0) 
    {
        _N = 0;
        newdata = malloc(sizeof(uint32_t)*1);
        newdata[0] = 0;
    }
    else
    {
        newdata = malloc(sizeof(uint32_t)*_N);
    }

    if(newdata==NULL)
    {
        ret = -2;
        return ret;
    }

    if(pnt->data==NULL)
    {
        pnt->data = malloc(sizeof(uint32_t)*1);
        pnt->data[0] = 0;
        pnt->N = 0;
    }

    if(pnt->data!=NULL)
    {
        for(I=0;I<pnt->N && I<_N;I++)
        {
            newdata[I] = pnt->data[I];
        }
        for(I=pnt->N;I<_N;I++)
        {
            newdata[I] = 0;
        }
    }
    else
    {
        for(I=0;I<_N;I++)
        {
            newdata[I] = 0;
        }
    }

    if(pnt->data!=NULL) {free(pnt->data); pnt->data=NULL;}
    pnt->data = newdata;
    pnt->N = _N;
    ret = 1;

    return ret;
}

int amsclil1_ulint_msindex(amsclil1_ulint *pnt)
{
    int ret = 0;
    int I = 0;
    if(pnt==NULL) return ret;
    for(I=pnt->N-1;I>=0;I--)
    {
        if(pnt->data[I]!=0)
        {
            ret = I;
            break;
        }
    }
    return ret;
}

int amsclil1_ulint_shrinktofit(amsclil1_ulint *pnt)
{
    int ret = 0;
    int msindex = 0;
    
    if(pnt==NULL) return -1;
    msindex = amsclil1_ulint_msindex(pnt);
    ret = amsclil1_ulint_resize(pnt,msindex+1);

    return ret;
}

void amsclil1_ulint_printhex(amsclil1_ulint *pnt)
{
    int I = 0;

    if(pnt==NULL) return;
    printf("0x");
    for(I=pnt->N-1;I>=0;I--)
    {
        printf("%08x",pnt->data[I]);
    }
    return;
}

void amsclil1_ulint_printbits(amsclil1_ulint *pnt)
{
    int I = 0;
    int J = 0;

    if(pnt==NULL) return;

    printf("0b ");
    for(I=pnt->N-1;I>=0;I--)
    {
        for(J=31;J>=0;J--)
        {
            printf("%d", ((pnt->data[I]>>J) & 1) );
        }
        printf(" ");
    }
    return;
}

void amsclil1_ulint_clear(amsclil1_ulint *pnt)
{
    int I = 0;
    for(I=0;I<pnt->N;I++)
    {
        pnt->data[I] = 0;
    }
    return;
}

//If _N is greater than current size, expand to larger size, otherwise do nothing
int amsclil1_ulint_resizetomin(amsclil1_ulint *pnt, int _N)
{
    int ret = 0;
    if(pnt!=NULL)
    {
        ret = 1;
        if(_N>pnt->N)
        {
            ret = amsclil1_ulint_resize(pnt,_N);
        }
    }

    return ret;
}

int amsclil1_ulint_copy(amsclil1_ulint *dest, amsclil1_ulint *src)
{
    int ret = 0;
    int res = 0;
    int I;

    if(dest==NULL)
    {
        ret = -1;
        return ret;
    }

    if(src==NULL)
    {
        amsclil1_ulint_clear(dest);
        ret = -1;
        return ret;
    }

    if(dest==src)
    {
        ret = 1;
        return ret;
    }

    res = amsclil1_ulint_resizetomin(dest,src->N);
    if(res<0)
    {
        ret = -1;
        return ret;
    }
    amsclil1_ulint_clear(dest);

    for(I=0;I<src->N;I++)
    {
        dest->data[I] = src->data[I];
    }

    return ret;
}

void amsclil1_ulint_tests1()
{
    amsclil1_ulint *a = NULL;
    amsclil1_ulint *b = NULL;
    

    amsclil1_ulint_new(&a);
    amsclil1_ulint_new(&b);
    
    amsclil1_ulint_resize(a,2);
    amsclil1_ulint_resize(b,2);

    a->data[1] = 0x0000ffff;
    a->data[0] = 1;
    b->data[1] = 0x0000ff01;
    
    printf("a="); amsclil1_ulint_printhex(a); printf("\n");
    printf("b="); amsclil1_ulint_printbits(b); printf("\n");

    amsclil1_ulint_resize(a,1);
    amsclil1_ulint_copy(b,a);

    printf("a="); amsclil1_ulint_printhex(a); printf("\n");
    printf("b="); amsclil1_ulint_printbits(b); printf("\n");   
    

    amsclil1_ulint_delete(&a);
    amsclil1_ulint_delete(&b);

    return;
}